@@ -3,6 +3,7 @@
  *
  * Copyright 1993        ajh@gec-mrc.co.uk (Andrew Haylett)
  * Copyright 1994-1997   rubini@linux.it (Alessandro Rubini)
+ * Copyright (C) 1998	Ian Zimmerman <itz@rahul.net>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,6 +20,8 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ********/
 
+static char rcsid[] = "$Id: gpm.c,v 1.15 1999/02/19 05:51:49 itz Exp $";
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>        /* strerror(); ?!?  */
@@ -71,18 +74,26 @@ int opt_test=DEF_TEST;
 int opt_ptrdrag=DEF_PTRDRAG;
 int opt_kill=0;
 int opt_repeater=0, opt_double=0;
+char* opt_repeater_type = 0;
+int opt_aged = 0;
 char *opt_special=NULL; /* special commands, like reboot or such */
+int opt_rawrep=0;
+Gpm_Type *repeated_type=0;
+
 char *consolename = "/dev/tty0";
 static int opt_resize=0; /* not really an option */
 char *prgname;
 struct winsize win;
 int maxx, maxy;
-int fifofd;
+int fifofd=-1;
 
 int eventFlag=0;
 Gpm_Cinfo *cinfo[MAX_VC+1];
 fd_set selSet, readySet, connSet;
 
+time_t last_selection_time;
+time_t opt_age_limit = 0;
+
 /*===================================================================*/
 /*
  *      first, all the stuff that used to be in gpn.c 
@@ -154,10 +165,15 @@ static inline void selection_copy(int x1, int y1, int x2, int y2, int mode)
 
   if ((fd=open_console(O_WRONLY))<0)
     oops("open_console");
-  PDEBUGG((stderr,"ctl %i, mode %i\n",(int)*buf,arg[4]));
+  gpm_debug_log(LOG_DEBUG,"ctl %i, mode %i",(int)*buf, arg[4]);
   if (ioctl(fd, TIOCLINUX, buf+sizeof(short)-1) < 0)
     oops("ioctl(TIOCLINUX)");
   close(fd);
+
+  if (mode < 3) {
+    opt_aged = 0;
+    last_selection_time = time(0);
+  } /*if*/
 }
 
 
@@ -167,6 +183,16 @@ static inline void selection_paste(void)
   char c=3;
   int fd;
 
+  if (!opt_aged && (0 != opt_age_limit) &&
+      (last_selection_time + opt_age_limit < time(0))) {
+    opt_aged = 1;
+  } /*if*/
+
+  if (opt_aged) {
+    gpm_debug_log(LOG_DEBUG,"Skipping paste; selection has aged");
+    return;
+  } 
+
   fd=open_console(O_WRONLY);
   if (ioctl(fd, TIOCLINUX, &c) < 0)
      oops("ioctl(TIOCLINUX)");
@@ -265,7 +291,7 @@ static inline int do_client(Gpm_Cinfo *cinfo, Gpm_Event *event)
 /*-------------------------------------------------------------------*/
 /*-------------------------------------------------------------------*/
 /*-------------------------------------------------------------------*/
-static inline char *getMouseData(int fd, Gpm_Type *type)
+static inline char *getMouseData(int fd, Gpm_Type *type, int kd_mode)
 {
   static unsigned char data[32]; /* quite a big margin :) */
   char *edata=data+type->packetlen;
@@ -276,18 +302,22 @@ static inline char *getMouseData(int fd, Gpm_Type *type)
     if (read(fd, data, howmany)!=howmany)
       {
       if (opt_test) exit(0);
-      LOG(("Error in read()ing first"));
+      gpm_debug_log(LOG_ERR, "Error in read()ing first: %s", strerror(errno));
       return NULL;
       }
+
+      if (kd_mode!=KD_TEXT && fifofd != -1 && opt_rawrep)
+	write(fifofd, data, howmany);
+
     if ((data[0]&(m_type->proto)[0]) != (m_type->proto)[1])
       {
       if (m_type->getextra)
 	{
 	data[1]=GPM_EXTRA_MAGIC_1; data[2]=GPM_EXTRA_MAGIC_2;
-	LOG(("Extra %02x",data[0]));
+        gpm_debug_log(LOG_DEBUG,"Extra %02x",data[0]);
 	return data;
 	}
-      LOG(("Error in protocol"));
+      gpm_debug_log(LOG_ERR,"Error in protocol");
       return NULL;
       }
 
@@ -302,22 +332,24 @@ static inline char *getMouseData(int fd, Gpm_Type *type)
       do
 	{
 	j=read(fd,edata-i,i); /* edata is pointer just after data */
+	if (kd_mode!=KD_TEXT && fifofd != -1 && opt_rawrep && j > 0)
+	  write(fifofd, edata-i, j);
 	i-=j;
 	}
       while (i&&j);
 
     if (i)
       {
-      LOG(("Error in read()ing rest"));
+      gpm_debug_log(LOG_ERR,"Error in read()ing rest: %s", strerror(errno));
       return NULL;
       }
     
     if ((data[1]&(m_type->proto)[2]) != (m_type->proto)[3])
       {
-      LOG(("Skipping a data packet (?)"));
+      gpm_debug_log(LOG_NOTICE,"Skipping a data packet (?)");
       return NULL;
       }
-  LOG(("Data %02x %02x %02x (%02x)",data[0],data[1],data[2],data[3]));
+  gpm_debug_log(LOG_DEBUG,"Data %02x %02x %02x (%02x)",data[0],data[1],data[2],data[3]);
   return data;
 }
 
@@ -351,12 +383,7 @@ static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
     {
     eventFlag=0;
 
-    if (!(m_type->absolute))
-      {
-      event->dx=nEvent.dx;
-      event->dy=nEvent.dy;
-      }
-    else /* absolute: a pen */
+    if (m_type->absolute)       /* a pen or other absolute device */
       {
       event->x=nEvent.x;
       event->y=nEvent.y;
@@ -374,7 +401,7 @@ static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
 
     do /* cluster loop */
       {
-      if (   ((data=getMouseData(fd,m_type))==NULL)
+      if (   ((data=getMouseData(fd,m_type,kd_mode))==NULL)
 	  || ((*(m_type->fun))(&nEvent,data)==-1) )
 	{
 	if (!i) return 0;
@@ -425,13 +452,22 @@ static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
 
   if (kd_mode!=KD_TEXT)
     {
-    signed char buffer[5];
-
-    /* sluggish... */
-    buffer[0]=(event->buttons ^ 0x07) | 0x80;
-    buffer[3] =  event->dx - (buffer[1] = event->dx/2); /* Markus */
-    buffer[4] = -event->dy - (buffer[2] = -event->dy/2);
-    write(fifofd,buffer,5);
+    if (fifofd != -1 && ! opt_rawrep)
+      {
+        if (m_type->absolute) /* hof Wed Feb  3 21:43:28 MET 1999 */ 
+          { /*prepare the values from a absolute device for repeatermode */
+             static struct timeval rept1,rept2;
+             gettimeofday(&rept2, (struct timezone *)NULL);
+             if (((rept2.tv_sec -rept1.tv_sec) *1000+(rept2.tv_usec-rept1.tv_usec)/1000)>250) 
+                { event->dx=0; event->dy=0; }
+             rept1=rept2;
+             
+             event->dy=event->dy*((win.ws_col/win.ws_row)+1);
+	     event->x=nEvent.x; 
+	     event->y=nEvent.y;
+          }
+        repeated_type->repeat_fun(event, fifofd); /* itz Mon Jan 11 23:39:44 PST 1999 */
+      }
     return 0; /* no events nor information for clients */
     }
 
@@ -533,10 +569,11 @@ static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
 
   event->margin=m;
 
-  LOG(("M: %3i %3i (%3i %3i) - butt=%i vc=%i cl=%i",event->dx,event->dy,
+  gpm_debug_log(LOG_DEBUG,"M: %3i %3i (%3i %3i) - butt=%i vc=%i cl=%i",event->dx,event->dy,
                                       event->x,event->y,
                                       event->buttons, event->vc,
-                                      event->clicks));
+                                      event->clicks);
+
   /* update the global state */
   statusX=event->x; statusY=event->y;
 
@@ -556,26 +593,48 @@ static int get_data(Gpm_Connect *where, int whence)
 
 #ifdef GPM_USE_MAGIC
   while ((i=read(whence,&check,sizeof(int)))==4 && check!=GPM_MAGIC)
-    LOG(("No magic"));
+    gpm_debug_log(LOG_NOTICE, "No magic");
 
   if (!i) return 0;
 
   if (check!=GPM_MAGIC)
     {
-    LOG(("Nothing more"));
+    gpm_debug_log(LOG_NOTICE, "Nothing more");
     return -1;
     }
 #endif
 
   if ((i=read(whence, where, sizeof(Gpm_Connect)))!=sizeof(Gpm_Connect))
     {
-    LOG(("No data"));
+    gpm_debug_log(LOG_NOTICE, "No data");
     return i ? -1 : 0;
     }
 
   return 1;
 }
 
+/* itz Sat Sep 12 20:46:51 PDT 1998 this used to be shared with
+   gpn.c.  Mixing apples and oranges. */
+
+static void
+xfer_options(int fd)
+{
+  Opt_struct_type opts;
+  struct mouse_features *which_mouse=mouse_table+1; /* local */
+
+  read(fd,&opts,sizeof(opts));
+  opt_accel=opts.a;
+  sprintf(opt_sequence,"%08d",opts.B);
+  opt_delta=opts.d;
+  opt_time=opts.i; opt_ptrdrag=opts.p; opt_scale=opts.r;
+  gpm_debug_level = opts.V;
+  opt_aged = opts.A;
+  gpm_debug_log(LOG_INFO,"got %i,%i,%i,%i,%i,%i,%i,%i", opt_accel, opts.B,
+                opt_delta, opt_time, opt_ptrdrag, opt_scale,
+                gpm_debug_level, opt_aged);
+  kill(getpid(),SIGWINCH); /* update delta's */
+}
+
 /*-------------------------------------------------------------------*/
                                  /* returns -1 if closing connection */
 static inline int processRequest(Gpm_Cinfo *ci, int vc) 
@@ -586,7 +645,7 @@ static inline int processRequest(Gpm_Cinfo *ci, int vc)
   static Gpm_Event event;
   static struct vt_stat stat;
 
-  PDEBUGG((stderr,"Request on %i (console %i)\n",fd,vc));
+  gpm_debug_log(LOG_INFO,"Request on %i (console %i)", ci->fd, vc);
   if (vc>MAX_VC) return -1;
 
   /* itz 10-22-96 this shouldn't happen now */
@@ -596,7 +655,7 @@ static inline int processRequest(Gpm_Cinfo *ci, int vc)
 
   if (!i) /* no data */
     {
-    LOG(("Closing"));
+    gpm_debug_log(LOG_INFO,"Closing");
     close(ci->fd);
     FD_CLR(ci->fd,&connSet);
     FD_CLR(ci->fd,&readySet);
@@ -638,27 +697,27 @@ static inline int processRequest(Gpm_Cinfo *ci, int vc)
   switch(conn.vc)
     {
     case GPM_REQ_SNAPSHOT:
-        i=open_console(O_RDONLY);
-	ioctl(i,VT_GETSTATE,&stat);
-	event.modifiers=6; /* code for the ioctl */
-	if (ioctl(i,TIOCLINUX,&(event.modifiers))<0)
-	  oops("get_shift_state");
-	close(i);
-	event.vc = stat.v_active;
-        event.x=statusX;   event.y=statusY;
-        event.dx=maxx;     event.dy=maxy;
-        event.buttons= statusB;
-        event.clicks=statusC;
-	/* fall through */
+      i=open_console(O_RDONLY);
+      ioctl(i,VT_GETSTATE,&stat);
+      event.modifiers=6; /* code for the ioctl */
+      if (ioctl(i,TIOCLINUX,&(event.modifiers))<0)
+        oops("get_shift_state");
+      close(i);
+      event.vc = stat.v_active;
+      event.x=statusX;   event.y=statusY;
+      event.dx=maxx;     event.dy=maxy;
+      event.buttons= statusB;
+      event.clicks=statusC;
+      /* fall through */
 
     case GPM_REQ_BUTTONS:
-        event.type= (opt_three==1 ? 3 : 2); /* buttons */
-        write(ci->fd,&event,sizeof(Gpm_Event));
-	break;
+      event.type= (opt_three==1 ? 3 : 2); /* buttons */
+      write(ci->fd,&event,sizeof(Gpm_Event));
+      break;
 
     case GPM_REQ_CONFIG:
-	xfer_options(1,ci->fd);
-	break;
+      xfer_options(ci->fd);
+      break;
     }
 
   return 0;
@@ -677,6 +736,10 @@ static inline int processConn(int fd) /* returns newfd or -1 */
   size_t len;
 #endif /* __GLIBC__ */
   struct sockaddr_un addr; /* reuse this each time */
+  struct stat statbuf;
+  time_t staletime;
+  uid_t uid;
+  char tty[20];
 
 /*....................................... Accept */
 
@@ -686,11 +749,11 @@ static inline int processConn(int fd) /* returns newfd or -1 */
   len=sizeof(addr);
   if ((newfd=accept(fd,(struct sockaddr *)&addr, &len))<0)
     {
-    LOG(("accept() failed: %s",strerror(errno)));
+    gpm_debug_log(LOG_ERR,"accept() failed: %s",strerror(errno));
     return -1;
     }
 
-  LOG(("Connecting at fd %i",newfd));
+  gpm_debug_log(LOG_INFO,"Connecting at fd %i",newfd);
 
   info=malloc(sizeof(Gpm_Cinfo));
   if (!info) oops("malloc()");
@@ -704,10 +767,43 @@ static inline int processConn(int fd) /* returns newfd or -1 */
 
   if ((vc=request->vc)>MAX_VC)
     {
-    LOG(("Request on vc %i > %i",vc,MAX_VC));
+    gpm_debug_log(LOG_WARNING,"Request on vc %i > %i", vc, MAX_VC);
     free(info);
     return -1;
-    }
+  }
+
+  if (stat (addr.sun_path, &statbuf) == -1 ||
+      !S_ISSOCK(statbuf.st_mode)) {
+    gpm_debug_log (LOG_ERR, "Address %s not a socket in processConn",addr.sun_path);
+    free(info);			/* itz 10-12-95 verify client's right */
+    return -1;			/* to read requested tty */
+  } /*if*/
+  
+  unlink (addr.sun_path);	/* delete socket */
+
+  staletime = time(0) - 30;
+  if (statbuf.st_atime < staletime
+      || statbuf.st_ctime < staletime
+      || statbuf.st_mtime < staletime) {
+    gpm_debug_log (LOG_ERR, "Socket too old in processConn");
+    free (info);
+    return -1;			/* socket is ancient */
+  } /*if*/
+  
+  uid = statbuf.st_uid;		/* owner of socket */
+  if (uid != 0) {
+    sprintf (tty, "/dev/tty%d", vc);
+    if (-1 == stat (tty, &statbuf)) {
+      gpm_debug_log (LOG_ERR, "stat on %s fails in processConn", tty);
+      free (info);
+      return -1;
+    } /*if*/
+    if (uid != statbuf.st_uid) {
+      gpm_debug_log (LOG_WARNING, "Failed gpm connect attempt by uid %d", uid); /* SUSPECT!! */
+      free (info);
+      return -1;
+    } /*if*/    
+  } /*if*/
 
 /* register the connection information in the right place */
 
@@ -715,9 +811,9 @@ static inline int processConn(int fd) /* returns newfd or -1 */
   info->fd=newfd;
   cinfo[vc]=info;
 
-  PDEBUG((stderr,"Pid %i, vc %i, ev %02X, def %02X, minm %02X, maxm %02X\n",
+  gpm_debug_log(LOG_DEBUG,"Pid %i, vc %i, ev %02X, def %02X, minm %02X, maxm %02X",
        request->pid, request->vc, request->eventMask, request->defaultMask,
-       request->minMod, request->maxMod));
+       request->minMod, request->maxMod);
 
 /* if the client gets motions, give it the current position */
   if(request->eventMask & GPM_MOVE)
@@ -740,11 +836,11 @@ void get_console_size(Gpm_Event *ePtr)
   close(i);
   if (!win.ws_col || !win.ws_row)
     {
-    fprintf(stderr, "%s: zero screen dimension, assuming 80x25.\n",prgname);
+    gpm_debug_log(LOG_NOTICE,"zero screen dimension, assuming 80x25");
     win.ws_col=80; win.ws_row=25;
     }
   maxx=win.ws_col; maxy=win.ws_row;
-  PDEBUGG((stderr,"%i - %i\n",maxx,maxy));
+  gpm_debug_log(LOG_DEBUG,"%i - %i",maxx,maxy);
   statusX=ePtr->x=maxx/2; statusY=ePtr->y=maxy/2;
 
   for (i=1; i <= 1+opt_double; i++)
@@ -755,16 +851,26 @@ void get_console_size(Gpm_Event *ePtr)
      * has square cells. (An author-centric observation ;-)
      */
     opt_scaley=opt_scale*50*maxx/80/maxy;
-    PDEBUG((stderr,"x %i, y %i\n",opt_scale,opt_scaley));
+    gpm_debug_log(LOG_DEBUG,"x %i, y %i",opt_scale,opt_scaley);
     }
 }
 
+static void
+gpm_exited(void)
+{
+  gpm_debug_log(LOG_DEBUG,"Removing files %s and %s", GPM_NODE_PID, GPM_NODE_CTL);
+  if (gpm_log_daemon) {
+    unlink(GPM_NODE_PID);
+  } /*if*/
+  unlink(GPM_NODE_CTL);
+}
+
 /*-------------------------------------------------------------------*/
 int main(int argc, char **argv)
 {
-  int mousefd, ctlfd, newfd;
+  int ctlfd, newfd;
   struct sockaddr_un ctladdr;
-  int i, len, kd_mode;
+  int i, len, kd_mode, fd;
   struct   timeval timeout;
   int maxfd=-1;
   int pending;
@@ -775,8 +881,49 @@ int main(int argc, char **argv)
 
 /*....................................... parse command line */
 
-  mousefd=cmdline(argc, argv);
-  maxfd=max(mousefd,maxfd);
+  cmdline(argc, argv);
+  atexit(gpm_exited);
+
+  for (i=1; i <= 1+opt_double; i++)
+    {
+      which_mouse=mouse_table+i; /* used to access options */
+
+      /* open the device with ndelay, to catch a locked device */
+      if (opt_dev)
+        {
+          if (!strcmp(opt_dev,"-"))
+            fd=0;
+          else if ((fd=open(opt_dev,O_RDWR | O_NDELAY))<0)
+            { oops(opt_dev); }
+        }
+      else /* use "/dev/mouse" */
+        {
+          opt_dev = "/dev/mouse";
+          if ((fd=open(opt_dev,O_RDWR | O_NDELAY))<0)
+            { oops(opt_dev); }
+        }
+
+      /* and then reset the flag */
+      fcntl(fd,F_SETFL,fcntl(fd,F_GETFL) & ~O_NDELAY);
+
+      /* init the device, and change mouse type */
+      if (m_type->init)
+        m_type=(m_type->init)(fd, m_type->flags, m_type);
+      if (!m_type)
+        { oops("mouse initialization failed"); }
+
+      if (opt_toggle)
+        {
+          unsigned int modem_lines;
+
+          ioctl(fd, TIOCMGET, &modem_lines);
+          modem_lines &= ~opt_toggle;
+          ioctl(fd, TIOCMSET, &modem_lines);
+        }
+
+      which_mouse->fd=fd;
+      maxfd=max(fd, maxfd);
+    } /*for*/
 
 /*....................................... catch interesting signals */
 
@@ -824,7 +971,7 @@ int main(int argc, char **argv)
     FD_SET(mouse_table[2].fd,&connSet);
 
   readySet=connSet;
-  FD_SET(mousefd,&readySet);
+  FD_SET(mouse_table[1].fd,&readySet);
 
   signal(SIGPIPE,SIG_IGN);  /* WARN */
 
@@ -845,7 +992,6 @@ int main(int argc, char **argv)
     else
       while ((pending=select(maxfd+1,&selSet, NULL_SET,NULL_SET,&timeout))==0)
 	{
-	LOGG(("Hi"));
 	selSet=readySet;
 	resetTimeout();
 	} /* go on */
@@ -870,13 +1016,13 @@ int main(int argc, char **argv)
     if (pending<0)
       {
       if (errno==EBADF) oops("select()");
-      LOG(("select(): %s",strerror(errno)));
+      gpm_debug_log(LOG_ERR,"select(): %s",strerror(errno));
       selSet=readySet;
       resetTimeout();
       continue;
       }
 
-    LOGG(("selected %i times",pending));
+    gpm_debug_log(LOG_DEBUG,"selected %i times",pending);
 
 /*....................................... manage graphic mode */
 
@@ -891,10 +1037,10 @@ int main(int argc, char **argv)
     close(fd);
     if (kd_mode != KD_TEXT && !opt_repeater)
       {
-      wait_text(&mousefd);
-      maxfd=max(maxfd,mousefd);
+      wait_text(&mouse_table[1].fd);
+      maxfd=max(maxfd,mouse_table[1].fd);
       readySet=connSet;
-      FD_SET(mousefd,&readySet);
+      FD_SET(mouse_table[1].fd,&readySet);
       continue; /* reselect */
       }
     }
@@ -910,7 +1056,6 @@ int main(int argc, char **argv)
       if (FD_ISSET(which_mouse->fd,&selSet))
 	  {
 	  FD_CLR(which_mouse->fd,&selSet); pending--;
-	  LOGG(("Got mouse"));
 	  if (processMouse(which_mouse->fd, &event, m_type, kd_mode))
 	    /*
 	     * pass it to the client, if any
@@ -928,7 +1073,6 @@ int main(int argc, char **argv)
     if (pending && FD_ISSET(ctlfd,&selSet))
       {
       FD_CLR(ctlfd,&selSet); pending--;
-      PDEBUGG((stderr,"Connection\n"));
       newfd=processConn(ctlfd);
       if (newfd>=0)
 	{
@@ -950,8 +1094,19 @@ int main(int argc, char **argv)
         if (FD_ISSET(ci->fd,&selSet)) 
 	  {
           FD_CLR(ci->fd,&selSet); pending--;
+
+          /* itz Sat Sep 12 21:10:22 PDT 1998 */
+          /* this code is clearly incorrect; the next highest
+             descriptor after the one we're closing is not necessarily
+             being used.  Fortunately, it doesn't hurt simply to leave this
+             out. */
+
+#ifdef NOTDEF
           if ((processRequest(ci,i)==-1) && maxfd==ci->fd)
             maxfd--;
+#else
+          (void)processRequest(ci,i);
+#endif
 	  }
 	}
       }
@@ -966,7 +1121,7 @@ int main(int argc, char **argv)
 	{
         FD_CLR(i,&selSet);
         pending--;
-        LOG(("Data on strange file descriptor %d", i));
+        gpm_debug_log(LOG_WARNING,"Data on strange file descriptor %d", i);
 	}
       }
 	     
@@ -981,18 +1136,13 @@ static void gpm_killed(int signo)
 {
   if (signo==SIGWINCH)
     {
-    PDEBUGG((stderr,"%s pid %i is resizing :-)\n",prgname,getpid()));
+    gpm_debug_log(LOG_INFO,"%s pid %i is resizing :-)", prgname, getpid());
     opt_resize++;
     return;
     }
   if (signo==SIGUSR1)
-    PDEBUG((stderr,"%s pid %i killed by a new %s\n",prgname,getpid(),prgname));
-  else
-    {
-    PDEBUGG((stderr,"Removing files...\n"));
-    unlink(GPM_NODE_PID);
-    unlink(GPM_NODE_CTL);
-    }
+    gpm_debug_log(LOG_INFO,"%s pid %i killed by a new %s",
+                  prgname, getpid(), prgname);
   exit(0);
 }
 
