@@ -2,6 +2,7 @@
  * gpm-Linux configuration file (server only)
  *
  * Copyright 1994-1996   rubini@linux.it
+ * Copyright (C) 1998 	Ian Zimmerman <itz@rahul.net>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -18,24 +19,27 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ********/
 
-#define GPM_NAME    "gpm-Linux"
-#define GPM_RELEASE "1.14"
-#define GPM_DATE    "June 1998"
+#ifndef _GPMCFG_INCLUDED
+#define _GPMCFG_INCLUDED
 
-/* Is logging enabled? */
-#undef CONFIG_GPM_LOG
+/* $Id: gpmCfg.h,v 1.6 1999/01/03 21:02:51 itz Exp $ */
+
+#define GPM_NAME    "gpm-Linux"
+#define GPM_DATE    "$Date: 1999/01/03 21:02:51 $"
 
 /* timeout for the select() syscall */
 #define SELECT_TIME 86400 /* one day */
 
-/* How many virtual consoles are managed? */
-#if 1
-#  define MAX_VC 64 /* this is always sure */
-#else
-#  include <linux/tty.h>
-#  define MAX_VC    MAX_NR_CONSOLES  /* doesn't work before 1.3.77 */
+#ifdef HAVE_LINUX_TTY_H
+#include <linux/tty.h>
 #endif
 
+/* How many virtual consoles are managed? */
+#ifndef MAX_NR_CONSOLES
+#  define MAX_NR_CONSOLES 64 /* this is always sure */
+#endif
+
+#define MAX_VC    MAX_NR_CONSOLES  /* doesn't work before 1.3.77 */
 
 /* How many buttons may the mouse have? */
 /* #define MAX_BUTTONS 3  ===> not used, it is hardwired :-( */
@@ -59,3 +63,5 @@
 #define DEF_PTRDRAG          1    /* double or triple click */
 #define DEF_TOGGLE           0
 #define DEF_GLIDEPOINT_TAP   0    /* tapping emulates no buttons by default */
+
+#endif /* _GPMCFG_INCLUDED */
