@@ -2,6 +2,7 @@
  * gpmInt.h - server-only stuff for gpm-Linux
  *
  * Copyright 1994,1995   rubini@linux.it (Alessandro Rubini)
+ * Copyright (C) 1998	Ian Zimmerman <itz@rahul.net>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -18,6 +19,13 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ********/
 
+#ifndef _GPMINT_INCLUDED
+#define _GPMINT_INCLUDED
+
+/* $Id: gpmInt.h,v 1.11 1999/01/12 07:42:53 itz Exp $ */
+
+#include <sys/types.h>          /* time_t */
+
 #include "gpmCfg.h"
 #include "gpm.h"
 
@@ -31,16 +39,6 @@
 
 #include "wd.h" /* when debugging macros */
 
-/*....................................... Logging */
-#ifdef CONFIG_GPM_LOG
-#  define LOG(args) Log args
-extern int Log(char *fmt, ...);
-#else
-#  define LOG(args) 
-#endif
-/* a placeholder */
-#define LOGG(args) 
-
 /*....................................... Strange requests (iff conn->pid==0)*/
 
 #define GPM_REQ_SNAPSHOT 0
@@ -66,6 +64,9 @@ typedef struct Gpm_Type {
   int howmany;            /* howmany bytes to read at a time */
   int getextra;           /* does it get an extra byte? (only mouseman) */
   int absolute;           /* flag indicating absolute pointing device */
+
+  int (*repeat_fun)(Gpm_Event *state, int fd); /* repeat this event into fd */
+                          /* itz Mon Jan 11 23:27:54 PST 1999 */
 }                   Gpm_Type;
 
 #define GPM_EXTRA_MAGIC_1 0xAA
@@ -92,6 +93,8 @@ struct mouse_features {
 
 extern struct mouse_features mouse_table[3], *which_mouse; /*the current one*/
 
+typedef struct Opt_struct_type {int a,B,d,i,p,r,V,A;} Opt_struct_type;
+
 #define opt_type     (which_mouse->opt_type)
 #define opt_dev      (which_mouse->opt_dev)
 #define opt_sequence (which_mouse->opt_sequence)
@@ -118,11 +121,16 @@ extern char *opt_lut;
 extern int opt_test, opt_ptrdrag;
 extern int opt_kill;
 extern int opt_repeater, opt_double;
+extern char* opt_repeater_type;
 extern int opt_kernel, opt_explicittype;
+extern int opt_aged;
+extern time_t opt_age_limit;
 extern char *opt_special;
+extern int opt_rawrep;
 extern int fifofd;
 extern char *consolename; /* the selected one */
 
+extern Gpm_Type *repeated_type;
 extern Gpm_Type mice[];
 extern struct winsize win;
 extern int maxx, maxy;
@@ -133,10 +141,7 @@ extern Gpm_Cinfo *cinfo[MAX_VC+1];
 int main(int argc, char **argv);
 
        /* gpn.c */
-int Log(char *fmt,...);              /* the logging utility               */
-int _oops(char *f, int n, char *s,int err);  /* printf's and exit(1)      */
-int xfer_options(int direction, int fd);
-#define oops(s) _oops(__FILE__, __LINE__,(s),errno)
+#define oops(s) gpm_oops(__FILE__, __LINE__,(s))
 int cmdline(int argc, char **argv);
 int giveInfo(int request, int fd);
        /* mice.c */
@@ -157,3 +162,4 @@ int twiddler_key_init(void);
 #define MAGIC_P(code) 
 #endif
 
+#endif /* _GPMINT_INCLUDED */
