@@ -27,7 +27,18 @@
 #include <sys/types.h>     /* socket() */
 
 #include "gpmInt.h"
-#include <curses.h> /* it is right? */
+
+#ifdef HAVE_NCURSES_H
+#include <ncurses.h>
+#else
+#ifdef HAVE_NCURSES_CURSES_H
+#include <ncurses/curses.h>
+#else
+#ifdef HAVE_CURSES_H
+#include <curses.h>
+#endif /* HAVE_CURSES_H */
+#endif /* HAVE_NCURSES_CURSES_H */
+#endif /* HAVE_NCURSES_H */
 
 #define GET(win) ((win) ? wgetch(win) : getch())
 
@@ -79,15 +90,22 @@ static Gpm_Event ev;
 #define DELAY_MS 100
     static struct timeval to={0,DELAY_MS*1000};
     static fd_set selSet;
-    static int prevchar=EOF;
+/* JD patch 11/08/1998 */
+#define MAXNBPREVCHAR 4         /* I don't think more is usefull, JD */
+    static int nbprevchar=0, prevchar[MAXNBPREVCHAR];
     extern gpm_convert_event(char *data, Gpm_Event *event);
     int c; char mdata[4];
 
-    if ((c=prevchar)!=EOF)  /* if ungetc() didn't suffice... */
+/* JD patch 11/08/1998 */
+  if (nbprevchar)  /* if there are some consumed char ... */
+        return prevchar[--nbprevchar];
+/* if ungetc() didn't suffice... */
+/*    if ((c=prevchar)!=EOF)
       {
       prevchar=EOF;
       return c;
       }
+*/
 
     while(1)
       {
@@ -106,14 +124,21 @@ static Gpm_Event ev;
       if ((flag=select(fd+1,&selSet,(fd_set *)NULL,(fd_set *)NULL,&to))==0)
 	return c;
       if ((c=GET(win))!='[')
-	{ungetc(c,stdin); return 0x1B;}
+        {prevchar[nbprevchar++]=c; return 0x1B;}  /* patche par JD 11/08/1998 */
+	/* {ungetc(c,stdin); return 0x1B;} */
 
       /* '[': go on */
       FD_ZERO(&selSet); FD_SET(fd,&selSet); to.tv_usec=DELAY_MS*1000;
+
       if ((flag=select(fd+1,&selSet,(fd_set *)NULL,(fd_set *)NULL,&to))==0)
-	{ungetc(c,stdin); return 0x1B;}
+        {prevchar[nbprevchar++]=c; return 0x1B;}  /* patche par JD 11/08/1998 */
+	/* {ungetc(c,stdin); return 0x1B;} */
+
       if ((c=GET(win))!='M')
-	{ungetc(c,stdin);prevchar='['; return 0x1B;}
+	/* patche par JD 11/08/1998 NOTICE: prevchar is a lifo !*/
+        {prevchar[nbprevchar++]=c; prevchar[nbprevchar++]='['; return 0x1B;}
+	/* {ungetc(c,stdin);prevchar='['; return 0x1B;} */
+
 
       /* now, it surely is a mouse event */
 
