@@ -1,7 +1,7 @@
 /*
  * libhigh.c - client library - high level (gpm-Linux)
  *
- * Copyright 1994,1995   rubini@ipvvis.unipv.it (Alessandro Rubini)
+ * Copyright 1994,1995   rubini@linux.it (Alessandro Rubini)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -149,8 +149,8 @@ Gpm_Roi *roi=gpm_current_roi;
 		continue;
 	  if (roi->yMin > ePtr->y || roi->yMax < ePtr->y)
 		continue;
-	  if (roi->minMod & ePtr->modifiers < roi->minMod) continue;
-	  if (roi->maxMod & ePtr->modifiers < ePtr->modifiers) continue;
+	  if ((roi->minMod & ePtr->modifiers) < roi->minMod) continue;
+	  if ((roi->maxMod & ePtr->modifiers) < ePtr->modifiers) continue;
 	  break;
 	  }
 
