@@ -2,6 +2,7 @@
  * libxtra.c - client library - extra functions (gpm-Linux)
  *
  * Copyright 1994,1995   rubini@linux.it (Alessandro Rubini)
+ * Copyright (C) 1998    Ian Zimmerman <itz@rahul.net>
  * 
  * xterm management is mostly by Miguel de Icaza
  *
@@ -20,6 +21,8 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ********/
 
+static char rcsid[] = "$Id: libxtra.c,v 1.9 1999/03/31 08:05:25 itz Exp $";
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/time.h>
@@ -37,48 +40,47 @@
  */
 
 static char *gpml_ver_s=GPM_RELEASE;
-static int  gpml_ver_i;
+static int  gpml_ver_i = 0;
 
 char *Gpm_GetLibVersion(int *where)
 {
-int i,j,k=0;
+  int i,j,k=0;
 
-if (!gpml_ver_i)
-  {
-  sscanf(gpml_ver_s,"%d.%d.%d",&i,&j,&k);
-  gpml_ver_i=i*10000+j*100+k;
-  }
-if (where) *where=gpml_ver_i;
-return gpml_ver_s;
+  if (!gpml_ver_i)
+    {
+      sscanf(gpml_ver_s,"%d.%d.%d",&i,&j,&k);
+      gpml_ver_i=i*10000+j*100+k;
+    }
+  if (where) *where=gpml_ver_i;
+  return gpml_ver_s;
 }
 
 static char gpm_ver_s[16];
-static int  gpm_ver_i;
+static int  gpm_ver_i = 0;
 
 char *Gpm_GetServerVersion(int *where)
 {
-char line[128];
-FILE *f;
-int i,j,k=0;
+  char line[128];
+  FILE *f;
+  int i,j,k=0;
 
-if (!gpm_ver_s[0])
-  {
-  f=popen("gpm -v","r");
-  if (!f) return NULL;
-  fgets(line,128,f);
-  pclose(f);
-  sscanf(line,"%*s %s",gpm_ver_s);  /* "gpm-Linux 0.98, March 1995" */
-  gpm_ver_s[strlen(gpm_ver_s)-1]='\0'; /* cut the ',' */
+  if (!gpm_ver_s[0])
+    {
+      f=popen(SBINDIR "/gpm -v","r");
+      if (!f) return NULL;
+      fgets(line,128,f);
+      if (pclose(f)) return 0;
+      sscanf(line,"%*s %s",gpm_ver_s);  /* "gpm-Linux 0.98, March 1995" */
+      gpm_ver_s[strlen(gpm_ver_s)-1]='\0'; /* cut the ',' */
 
-  sscanf(gpm_ver_s,"%d.%d.%d",&i,&j,&k);
-  gpm_ver_i=i*10000+j*100+k;
-  }
+      sscanf(gpm_ver_s,"%d.%d.%d",&i,&j,&k);
+      gpm_ver_i=i*10000+j*100+k;
+    }
   
-if (where) *where=gpm_ver_i;
+  if (where) *where=gpm_ver_i;
   return gpm_ver_s;
 }
 
-
 /*-------------------------------------------------------------------*/
 /*
  * This returns all the available information about the current situation:
@@ -87,43 +89,54 @@ if (where) *where=gpm_ver_i;
  */
 int Gpm_GetSnapshot(Gpm_Event *ePtr)
 {
-Gpm_Connect conn;
-Gpm_Event event;
-fd_set sillySet;
-struct timeval to={0,0};
-int i;
+  Gpm_Connect conn;
+  Gpm_Event event;
+  fd_set sillySet;
+  struct timeval to={0,0};
+  int i;
 
   if (!gpm_ver_i)
-	{
-	Gpm_GetServerVersion(NULL);
-	PDEBUG((stderr,"libgpm: got server version as %i\n",gpm_ver_i));
-	}
-  if (gpm_ver_i<9802 || gpm_fd <=0)
-	{
-	PDEBUG((stderr,"libgpm: can't get status info\n"));
-	return -1; /* error */
-	}
+    {
+      if (0 == Gpm_GetServerVersion(NULL)) {
+        gpm_debug_log(LOG_WARNING,"can't get gpm server version");
+      } /*if*/
+      gpm_debug_log(LOG_INFO,"libgpm: got server version as %i",gpm_ver_i);
+    }
+  if (gpm_ver_i<9802)
+    {
+      gpm_debug_log(LOG_NOTICE,"gpm server version too old to obtain status info");
+      return -1; /* error */
+    }
+  if (gpm_fd <=0)
+    {
+      gpm_debug_log(LOG_NOTICE,"gpm connection must be open to obtain status info");
+      return -1; /* error */
+    }
 
   conn.pid=0; /* this signals a request */
   if (ePtr)
-	conn.vc=GPM_REQ_SNAPSHOT;
+    conn.vc=GPM_REQ_SNAPSHOT;
   else
-	{
-	conn.vc=GPM_REQ_BUTTONS;
-	ePtr=&event;
-	}
+    {
+      conn.vc=GPM_REQ_BUTTONS;
+      ePtr=&event;
+    }
 
   if (gpm_fd==-1) return -1;
   FD_ZERO(&sillySet);
   FD_SET(gpm_fd,&sillySet);
   if (select(gpm_fd+1,&sillySet,NULL,NULL,&to)==1)
-	return 0;
+    return 0;
   write(gpm_fd,&conn,sizeof(Gpm_Connect));
 
   if ((i=Gpm_GetEvent(ePtr))!=1)
-	return -1;
+    return -1;
 
   i=ePtr->type; ePtr->type=0;
   return i; /* number of buttons */
 }
   
+
+/* Local Variables: */
+/* c-indent-level: 2 */
+/* End: */
