@@ -2,6 +2,7 @@
  * mev.c - simple client to print mouse events (gpm-Linux)
  *
  * Copyright 1994,1995   rubini@linux.it (Alessandro Rubini)
+ * Copyright (C) 1998 Ian Zimmerman <itz@rahul.net>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -18,6 +19,8 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ********/
 
+static char rcsid[] = "$Id: mev.c,v 1.5 1999/03/31 07:59:38 itz Exp $";
+
 /*
  * This client is meant to be used both interactively to check
  * that gpm is working, and as a background process to convert gpm events
@@ -334,6 +337,12 @@ do_snapshot()
   int i=Gpm_GetSnapshot(&event);
   char *s;
 
+  if (-1 == i) {
+    fprintf(stderr,"Warning: cannot get snapshot!\n");
+    fprintf(stderr,"Have you run \"configure\" and \"make install\"?\n");
+    return;
+  } /*if*/
+
   fprintf(stderr,"Mouse has %d buttons\n",i);
   fprintf(stderr,"Currently sits at (%d,%d)\n",event.x,event.y);
   fprintf(stderr,"The window is %d columns by %d rows\n",event.dx,event.dy);
@@ -423,7 +432,7 @@ int main(int argc, char **argv)
 
   prgname=argv[0];
 
-  if (cmdline(argc,argv,"C:d:e:Efim:M:p:u"))
+  if (cmdline(argc,argv,"C:d:e:Efim:M:pu"))
     exit(usage());
 
   if (opt_vc) {
@@ -442,6 +451,9 @@ int main(int argc, char **argv)
   if (Gpm_Open(&conn,opt_vc) == -1) {
     fprintf(stderr,"%s: Can't open mouse connection\n",prgname);
     exit (1);
+  } else if (-2 == gpm_fd) {
+    fprintf(stderr,"%s: use rmev to see gpm events in xterm or rxvt\n",prgname); 
+    exit(2);
   } /*if*/
 
   my_handler= opt_emacs ? emacs_handler : user_handler;
