@@ -2,6 +2,13 @@
  * mouse-test.c 
  *
  * Copyright 1995   rubini@linux.it (Alessandro Rubini)
+ * Copyright (C) 1998 Ian Zimmerman <itz@rahul.net>
+ *
+ * Tue,  5 Jan 1999 23:15:23 +0000, modified by James Troup <james@nocrew.org>:
+ * (main): exclude devices with a minor number of 130 from
+ * the device probe to avoid causing spontaneous reboots on machines
+ * where watchdog is used.  Reported by Jim Studt <jim@federated.com>
+ * [Debian bug report #22602]
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -18,6 +25,8 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ********/
 
+static char rcsid[] = "$Id: mouse-test.c,v 1.5 1999/03/31 08:03:33 itz Exp $";
+
 /* This code is horrible. Browse it at your risk */
 
 #include <stdio.h>
@@ -87,14 +96,6 @@ printf("Note that this program is by no means complete, and its main role is\n"
 return 0;
 }
 
-int _oops(char *f, int n, char *s, int err)
-{
-  PDEBUG((stderr,"oops() invoked from %s(%i)\n",f,n));
-  fprintf(stderr,"%s: %s: %s\n",prgname,s,strerror(err));
-  exit(1);
-}
-
-
 /* I don't like curses, so I'm doing low level stuff here */
 
 #define GOTOXY(f,x,y)   fprintf(f,"\x1B[%03i;%03iH",y,x)
@@ -135,11 +136,14 @@ void killed(int signo)
   exit(0);
 }
 
-struct device **makedev(struct device **current, char *name)
+struct device **gpm_makedev(struct device **current, char *name)
 {
-  int fd;
-  if ((fd=open(name,O_RDWR))==-1)
+  int fd; int modes;
+  if ((fd=open(name,O_RDWR|O_NONBLOCK))==-1)
     { perror(name); return current; }
+  modes = fcntl(fd, F_GETFL);
+  if (0 > fcntl(fd, F_SETFL, modes & ~O_NONBLOCK))
+    { close(fd); perror(name); return current; }
 
   *current=malloc(sizeof(struct device));
   if (!*current) oops("malloc()");
@@ -275,21 +279,22 @@ int main(int argc, char **argv)
   /* init the list of possible devices */
 
   for (nextdev=&devlist, i=1; i<argc; i++)
-    nextdev=makedev(nextdev,argv[i]);
+    nextdev=gpm_makedev(nextdev,argv[i]);
 
   if (argc==1) /* no cmdline, get all devices */
     {
     FILE *f;
     char s[64];
 
+    /* Skip 10, 130 (== watchdog) to avoid causing spontaneous reboots */
     f=popen("ls -l /dev/* | "
-	    "awk '/^c/ && $5==\"10,\" || $NF ~ \"ttyS[0-3]$\" {print $NF}'",
+	    "awk '/^c/ && $5==\"10,\" && $6!=\"130\" || $NF ~ \"ttyS[0-3]$\" {print $NF}'",
 	    "r");
     if (!f) oops("popen()");
     while (fgets(s,64,f))
       {
       s[strlen(s)-1]='\0'; /* trim '\n' */
-      nextdev=makedev(nextdev,s);
+      nextdev=gpm_makedev(nextdev,s);
       }
     pclose(f);
     }
